// -*- C -*-
/*!
 * @file  rtc_msg.h
 * @brief Low level message handling functions
 * @date  $Date$
 *
 * $Id$
 */
#ifndef RTC_MSG_H
#define RTC_MSG_H

//============================================================
// Message handling
//============================================================
#define DATA_MAX    16
#define HEADER_LEN   4
#define PARITY_LEN   1
#define MSG_MAX_LEN 21
#define BUFFER_SIZE 24


//------------------------------------------------------------
// Message struct
//------------------------------------------------------------

BYTE g_Buffer[BUFFER_SIZE];  // Mobt@
BYTE g_NextIn = 0;           // ݃|C^
BYTE g_NextOut = 0;          // ǂݏo|C^
BYTE g_MsgBegin = 0;         // bZ[Wn܂
BYTE g_MsgEnd = 0;           // bZ[WI
BYTE g_MsgSize = 0;          // bZ[WTCY

typedef struct {
  unsigned int sum;
  unsigned int cnt;
} outmsg_t;

outmsg_t g_OutMsg;

#define bkbhit (g_NextIn != g_NextOut)


// In message handling funcs
BYTE bgetc();
void rtc_msg_read();
int rtc_msg_parity_check();
unsigned int rtc_msg_get(int i);

// Out message handling funcs
void rtc_msg_begin();
void rtc_msg_putc(BYTE data);
unsigned int rtc_msg_end();


//============================================================
// RS232C interrupt handler 
//============================================================
#int_rda
void serial_isr()
{
  int t;
  
  g_Buffer[g_NextIn] = getc();
  t = g_NextIn;
  g_NextIn = (g_NextIn + 1) % BUFFER_SIZE;
  if (g_NextIn == g_NextOut)
    {
      g_NextIn=t;           // Buffer full !!
    }
}


//============================================================
// bgetc(): Buffered getc
//============================================================
BYTE bgetc()
{
   BYTE c;

   while(!bkbhit) ;
   c = g_Buffer[g_NextOut];
   g_NextOut = (g_NextOut + 1) % BUFFER_SIZE;
   return c;
}



//============================================================
// rtc_msg_read(): Read message from buffer
//============================================================
#define RS_NO_MORE_DATA 0
#define RS_EXIST_DATA   1
void rtc_msg_read()
{
  int index;
  long int time_out;
  BYTE c;

  index = 0;
  time_out = 0;
  // skip data until header (0xff) appears in received message
  while (1)
    {
      c = bgetc();

      if (c == 0xFF)
        {
          g_MsgBegin = g_NextOut - 1;
          index++;
          break;
        }
    }

  // increment read pointer of buffer
  while (1)
    {
      if (bkbhit)
        {
          time_out = 0;
          c = bgetc();
          if (index < MSG_MAX_LEN)
            {
              index++;
            }
        }

      time_out++;

      if (time_out > 100) // timeout 100us
        {
		  g_MsgSize = index; //data size
		  g_MsgEnd = (BUFFER_SIZE + g_NextOut - 1) % BUFFER_SIZE;
          return;
        }
      delay_us(1);
    }
}


 //============================================================
// rtc_parity_check()
//============================================================
int rtc_msg_parity_check()
{
  int i;
  int parity = 0; // parity calculated from received data
  int end_mark;   // Received parity data

  /* calculate parity without header(0xFF)
   * Format:
   * |  0 |..............|size|
   * |0xFF|....data......| SUM|
   * SUM: parity of data field
   */
  // Calculate parity from received data except header and end_mark.
  for (i = 1; i < g_MsgSize - 1; i++)
    {
      parity ^= rtc_msg_get(i);
    }
  end_mark = rtc_msg_get(g_MsgSize - 1);

  return (end_mark == parity) ? TRUE : FALSE;
}


//============================================================
// rtc_msg_get(i): Get i-th data from buffer
//============================================================
unsigned int rtc_msg_get(int i)
{
  return g_Buffer[(g_MsgBegin + i) % BUFFER_SIZE];
}


//============================================================
// rtc_msg_begin(): Begining of message data stream
//============================================================
void rtc_msg_begin()
{
  g_OutMsg.sum = 0;
  g_OutMsg.cnt = 0;
  putc(0xFF); ++g_OutMsg.cnt;
}


//============================================================
// rtc_msg_putc(): Message putc
//============================================================
void rtc_msg_putc(BYTE data)
{
  putc(data);
  g_OutMsg.sum ^= data;
  g_OutMsg.cnt++;
}


//============================================================
// rtc_msg_end(): End of data streamu
//============================================================
unsigned int rtc_msg_end()
{
  unsigned int cnt;
  putc(g_OutMsg.sum);
  cnt = g_OutMsg.cnt;
  g_OutMsg.sum = 0;
  g_OutMsg.cnt = 0;
  putc(0x0D);
  putc(0x0A);
  return cnt;
}


#endif // RTC_MSG_H
