// -*- C -*-
/*!
 * @file  rtc_outport.c
 * @brief RTC-L OutPort handling function for Sample component
 * @date  Tue Oct  4 21:57:16 2005
 *
 * $Id$
 */

#ifndef RTC_OUTPORT_C
#define RTC_OUTPORT_C

#include "rtc_msg.h"
/* OutPort Data Format:
 * | 0  |     1      |     2    |        3       |  4  |..............|
 * |0xFF|OUTPORT_PUSH|OUTPORT_ID|OUTPORT_DATA_LEN|data0|data1|....|SUM|
 *
 */
#define OUTPORT_PUSH     1
#define OUTPORT_GET      1
#define OUTPORT_ID       2
#define OUTPORT_DATA_LEN 3
#define OUTPORT_DATA_POS 4


// <rtc-template block="outport_decl">
// Number of OutPort
#define OUTPORT_NUM 2

// OutPort data buffer array and InPort length list
char* g_outport_p[INPORT_NUM];
int   g_outport_len[INPORT_NUM];


// OutPort data declaration: Ctrl
#define CTRL_LEN 4
#define CTRL_ID  0
long g_CtrlOut[2];

// OutPort data declaration: Monitor
#define MONITOR_LEN 4
#define MONITOR_ID  1
long g_MonitorOut[2];


// </rtc-template>


// OutPort handling functions
void rtc_outport_init();
void rtc_parse_outport_cmd();
void rtc_outport_push(BYTE id);
void rtc_outport_write(BYTE id, BYTE len, BYTE* data);
void rtc_outport_begin(unsigned int id, unsigned int len);
void rtc_outport_putc(int data);
void rtc_outport_end();


// OutPort initialization
void rtc_outport_init()
{
// <rtc-template block="outport_init">


  g_outport_p[CTRL_ID]   = (char*)g_ctrlOut;
  g_outport_len[CTRL_ID] = CTRL_LEN;

  g_outport_p[MONITOR_ID]   = (char*)g_monitorOut;
  g_outport_len[MONITOR_ID] = MONITOR_LEN;

  g_CtrlOut[0] = 0;
  g_CtrlOut[1] = 255;
  g_MonitorOut[0] = 2;//test mode
  g_MonitorOut[1] = 2;
   
}


// OutPort command parse function
void rtc_parse_outport_cmd()
{
  int id;
  id = rtc_msg_get(OUTPORT_ID);
  if (id > (OUTPORT_NUM - 1)) 
	  {RTM_ACK_ERR;}
  else 
	 {rtc_outport_push(id);}

}


// OutPort data push
void rtc_outport_push(BYTE id)
{
  rtc_outport_write(id, g_outport_len[id], g_outport_p[id]);
}


// OutPort data write
void rtc_outport_write(BYTE id, BYTE len, BYTE* data)
{
//  BYTE sum;
  int i;

  rtc_msg_begin();
  rtc_msg_putc(OUTPORT_CMD);
  rtc_msg_putc(id);
  rtc_msg_putc(len);
  for (i = 0; i < len; ++i)
    {
      rtc_msg_putc(*data);
	  ++data;
    }
  rtc_msg_end();
}


#endif // end of rtc_outport.c
